module Appril
  class RTCPController < BaseController
    attr_reader :socket

    def index
      return unless websocket?

      @socket = Tubesock.hijack(env)
      @socket.onopen(&method(:on_open))
      @socket.onmessage(&method(:on_message))
      @socket.onclose(&method(:on_close))
      @socket.listen

      halt websocket_response
    end

    private

    def on_open
      connected
      write(serial: 0, data: _env)
    end

    def on_message msg

      msg = indifferent_params(JSON.parse(msg))
      if controller = resolve_controller(msg[:controller])

        status, _, body = call_controller(
          controller,
          msg[:method],
          msg[:arguments],
          msg[:params],
          env.merge(RocketIO::REQUEST_METHOD => msg[:request_method], rtcp_serial: msg[:serial])
        )

        if status.to_i / 100 == 2

          if body.is_a?(Proc)
            body.call
            return
          end

          if msg[:reply]
            write(serial: msg[:serial], data: body)
          end

          return
        end
      else
        body = '404: Not Found'
      end

      error = if body.is_a?(Array) && body.size == 1
        body[0]
      else
        body
      end

      if msg && msg[:serial]
        write(serial: msg[:serial], error: error)
      else
        write(error: error)
      end

    rescue Errno::EPIPE => e
      close
    rescue Exception => e
      __error__(500, e)
    end

    def on_close
      @socket.close! if @socket
      @socket = nil
      disconnected
    end

    def resolve_controller controller
      ::RocketIO.controllers.find {|c| c.name == controller}
    end

    def call_controller controller, method, arguments, params, env
      controller.new(method, arguments, params).call(env)
    end

    def write data
      @socket.send_data(data.to_json)
    end

    # called after socket connection established
    def connected
    end

    # environmental data to be sent to client at App initialization
    def _env
      {}
    end

    # called when socket connection closed
    def disconnected
    end
  end
end
